@@ -1,21 +1,19 @@
+// SPDX-License-Identifier: GPL-2.0
 /*
  * Defines interfaces for interacting wtih the Raspberry Pi firmware's
  * property channel.
  *
  * Copyright © 2015 Broadcom
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License version 2 as
- * published by the Free Software Foundation.
  */
 
 #include <linux/dma-mapping.h>
+#include <linux/kref.h>
 #include <linux/mailbox_client.h>
 #include <linux/module.h>
 #include <linux/of_platform.h>
 #include <linux/platform_device.h>
 #include <linux/reboot.h>
-#include <linux/workqueue.h>
+#include <linux/slab.h>
 #include <soc/bcm2835/raspberrypi-firmware.h>
 
 #define MBOX_MSG(chan, data28)		(((data28) & ~0xf) | ((chan) & 0xf))
@@ -23,46 +21,17 @@
 #define MBOX_DATA28(msg)		((msg) & ~0xf)
 #define MBOX_CHAN_PROPERTY		8
 
-#define UNDERVOLTAGE_BIT		BIT(0)
-
-
-/*
- * This section defines some rate limited logging that prevent
- * repeated messages at much lower Hz than the default kernel settings.
- * It's usually 5s, this is 5 minutes.
- * Burst 3 means you may get three messages 'quickly', before
- * the ratelimiting kicks in.
- */
-#define LOCAL_RATELIMIT_INTERVAL (5 * 60 * HZ)
-#define LOCAL_RATELIMIT_BURST 3
-
-#ifdef CONFIG_PRINTK
-#define printk_ratelimited_local(fmt, ...)	\
-({						\
-	static DEFINE_RATELIMIT_STATE(_rs,	\
-		LOCAL_RATELIMIT_INTERVAL,	\
-		LOCAL_RATELIMIT_BURST);		\
-						\
-	if (__ratelimit(&_rs))			\
-		printk(fmt, ##__VA_ARGS__);	\
-})
-#else
-#define printk_ratelimited_local(fmt, ...)	\
-	no_printk(fmt, ##__VA_ARGS__)
-#endif
-
-#define pr_crit_ratelimited_local(fmt, ...)              \
-	printk_ratelimited_local(KERN_CRIT pr_fmt(fmt), ##__VA_ARGS__)
-#define pr_info_ratelimited_local(fmt, ...)              \
-	printk_ratelimited_local(KERN_INFO pr_fmt(fmt), ##__VA_ARGS__)
-
+static struct platform_device *rpi_hwmon;
+static struct platform_device *rpi_clk;
 
 struct rpi_firmware {
 	struct mbox_client cl;
 	struct mbox_chan *chan; /* The property channel. */
 	struct completion c;
 	u32 enabled;
-	struct delayed_work get_throttled_poll_work;
+	u32 get_throttled;
+
+	struct kref consumers;
 };
 
 static struct platform_device *g_pdev;
@@ -79,7 +48,7 @@ static void response_callback(struct mbox_client *cl, void *msg)
  * Sends a request to the firmware through the BCM2835 mailbox driver,
  * and synchronously waits for the reply.
  */
-int
+static int
 rpi_firmware_transaction(struct rpi_firmware *fw, u32 chan, u32 data)
 {
 	u32 message = MBOX_MSG(chan, data);
@@ -91,8 +60,12 @@ rpi_firmware_transaction(struct rpi_firmware *fw, u32 chan, u32 data)
 	reinit_completion(&fw->c);
 	ret = mbox_send_message(fw->chan, &message);
 	if (ret >= 0) {
-		wait_for_completion(&fw->c);
-		ret = 0;
+		if (wait_for_completion_timeout(&fw->c, HZ)) {
+			ret = 0;
+		} else {
+			ret = -ETIMEDOUT;
+			WARN_ONCE(1, "Firmware transaction timeout");
+		}
 	} else {
 		dev_err(fw->cl.dev, "mbox_send_message returned %d\n", ret);
 	}
@@ -100,7 +73,6 @@ rpi_firmware_transaction(struct rpi_firmware *fw, u32 chan, u32 data)
 
 	return ret;
 }
-EXPORT_SYMBOL_GPL(rpi_firmware_transaction);
 
 /**
  * rpi_firmware_property_list - Submit firmware property list
@@ -152,7 +124,7 @@ int rpi_firmware_property_list(struct rpi_firmware *fw,
 		 * error, if there were multiple tags in the request.
 		 * But single-tag is the most common, so go with it.
 		 */
-		dev_dbg(fw->cl.dev, "Request 0x%08x returned status 0x%08x\n",
+		dev_err(fw->cl.dev, "Request 0x%08x returned status 0x%08x\n",
 			buf[2], buf[1]);
 		ret = -EINVAL;
 	}
@@ -180,108 +152,48 @@ EXPORT_SYMBOL_GPL(rpi_firmware_property_list);
 int rpi_firmware_property(struct rpi_firmware *fw,
 			  u32 tag, void *tag_data, size_t buf_size)
 {
-	/* Single tags are very small (generally 8 bytes), so the
-	 * stack should be safe.
-	 */
-	u8 data[buf_size + sizeof(struct rpi_firmware_property_tag_header)];
-	struct rpi_firmware_property_tag_header *header =
-		(struct rpi_firmware_property_tag_header *)data;
+	struct rpi_firmware_property_tag_header *header;
 	int ret;
 
+	/* Some mailboxes can use over 1k bytes. Rather than checking
+	 * size and using stack or kmalloc depending on requirements,
+	 * just use kmalloc. Mailboxes don't get called enough to worry
+	 * too much about the time taken in the allocation.
+	 */
+	void *data = kmalloc(sizeof(*header) + buf_size, GFP_KERNEL);
+
+	if (!data)
+		return -ENOMEM;
+
+	header = data;
 	header->tag = tag;
 	header->buf_size = buf_size;
 	header->req_resp_size = 0;
-	memcpy(data + sizeof(struct rpi_firmware_property_tag_header),
-	       tag_data, buf_size);
+	memcpy(data + sizeof(*header), tag_data, buf_size);
 
-	ret = rpi_firmware_property_list(fw, &data, sizeof(data));
-	memcpy(tag_data,
-	       data + sizeof(struct rpi_firmware_property_tag_header),
-	       buf_size);
+	ret = rpi_firmware_property_list(fw, data, buf_size + sizeof(*header));
+
+	memcpy(tag_data, data + sizeof(*header), buf_size);
+
+	kfree(data);
+
+	if ((tag == RPI_FIRMWARE_GET_THROTTLED) &&
+	     memcmp(&fw->get_throttled, tag_data, sizeof(fw->get_throttled))) {
+		memcpy(&fw->get_throttled, tag_data, sizeof(fw->get_throttled));
+		sysfs_notify(&fw->cl.dev->kobj, NULL, "get_throttled");
+	}
 
 	return ret;
 }
 EXPORT_SYMBOL_GPL(rpi_firmware_property);
 
-static int rpi_firmware_get_throttled(struct rpi_firmware *fw, u32 *value)
-{
-	static int old_firmware;
-	static ktime_t old_timestamp;
-	static u32 old_value;
-	u32 new_sticky, old_sticky, new_uv, old_uv;
-	ktime_t new_timestamp;
-	s64 elapsed_ms;
-	int ret;
-
-	if (!fw)
-		return -EBUSY;
-
-	if (old_firmware)
-		return -EINVAL;
-
-	/*
-	 * We can't run faster than the sticky shift (100ms) since we get
-	 * flipping in the sticky bits that are cleared.
-	 * This happens on polling, so just return the previous value.
-	 */
-	new_timestamp = ktime_get();
-	elapsed_ms = ktime_ms_delta(new_timestamp, old_timestamp);
-	if (elapsed_ms < 150) {
-		*value = old_value;
-		return 0;
-	}
-	old_timestamp = new_timestamp;
-
-	/* Clear sticky bits */
-	*value = 0xffff;
-
-	ret = rpi_firmware_property(fw, RPI_FIRMWARE_GET_THROTTLED,
-				    value, sizeof(*value));
-
-	if (ret) {
-		/* If the mailbox call fails once, then it will continue to
-		 * fail in the future, so no point in continuing to call it
-		 * Usual failure reason is older firmware
-		 */
-		old_firmware = 1;
-		dev_err(fw->cl.dev, "Get Throttled mailbox call failed");
-
-		return ret;
-	}
-
-	new_sticky = *value >> 16;
-	old_sticky = old_value >> 16;
-	old_value = *value;
-
-	/* Only notify about changes in the sticky bits */
-	if (new_sticky == old_sticky)
-		return 0;
-
-	new_uv = new_sticky & UNDERVOLTAGE_BIT;
-	old_uv = old_sticky & UNDERVOLTAGE_BIT;
-
-	if (new_uv != old_uv) {
-		if (new_uv)
-			pr_crit_ratelimited_local(
-				"Under-voltage detected! (0x%08x)\n",
-				 *value);
-		else
-			pr_info_ratelimited_local(
-				"Voltage normalised (0x%08x)\n",
-				 *value);
-	}
-
-	sysfs_notify(&fw->cl.dev->kobj, NULL, "get_throttled");
-
-	return 0;
-}
-
 static int rpi_firmware_notify_reboot(struct notifier_block *nb,
 				      unsigned long action,
 				      void *data)
 {
 	struct rpi_firmware *fw;
 	struct platform_device *pdev = g_pdev;
+	u32 reboot_flags = 0;
 
 	if (!pdev)
 		return 0;
@@ -290,38 +202,40 @@ static int rpi_firmware_notify_reboot(struct notifier_block *nb,
 	if (!fw)
 		return 0;
 
-	(void)rpi_firmware_property(fw, RPI_FIRMWARE_NOTIFY_REBOOT,
-				    0, 0);
+	// The partition id is the first parameter followed by zero or
+	// more flags separated by spaces indicating the reason for the reboot.
+	//
+	// 'tryboot': Sets a one-shot flag which is cleared upon reboot and
+	//            causes the tryboot.txt to be loaded instead of config.txt
+	//            by the bootloader and the start.elf firmware.
+	//
+	//            This is intended to allow automatic fallback to a known
+	//            good image if an OS/FW upgrade fails.
+	//
+	// N.B. The firmware mechanism for storing reboot flags may vary
+	// on different Raspberry Pi models.
+	if (data && strstr(data, " tryboot"))
+		reboot_flags |= 0x1;
+
+	// The mailbox might have been called earlier, directly via vcmailbox
+	// so only overwrite if reboot flags are passed to the reboot command.
+	if (reboot_flags)
+		(void)rpi_firmware_property(fw, RPI_FIRMWARE_SET_REBOOT_FLAGS,
+				&reboot_flags, sizeof(reboot_flags));
+
+	(void)rpi_firmware_property(fw, RPI_FIRMWARE_NOTIFY_REBOOT, NULL, 0);
 
 	return 0;
 }
 
-static void get_throttled_poll(struct work_struct *work)
-{
-	struct rpi_firmware *fw = container_of(work, struct rpi_firmware,
-					       get_throttled_poll_work.work);
-	u32 dummy;
-	int ret;
-
-	ret = rpi_firmware_get_throttled(fw, &dummy);
-
-	/* Only reschedule if we are getting valid responses */
-	if (!ret)
-		schedule_delayed_work(&fw->get_throttled_poll_work, 2 * HZ);
-}
-
 static ssize_t get_throttled_show(struct device *dev,
 				  struct device_attribute *attr, char *buf)
 {
 	struct rpi_firmware *fw = dev_get_drvdata(dev);
-	u32 value;
-	int ret;
 
-	ret = rpi_firmware_get_throttled(fw, &value);
-	if (ret)
-		return ret;
+	WARN_ONCE(1, "deprecated, use hwmon sysfs instead\n");
 
-	return sprintf(buf, "%x\n", value);
+	return sprintf(buf, "%x\n", fw->get_throttled);
 }
 
 static DEVICE_ATTR_RO(get_throttled);
@@ -338,34 +252,121 @@ static const struct attribute_group rpi_firmware_dev_group = {
 static void
 rpi_firmware_print_firmware_revision(struct rpi_firmware *fw)
 {
+	time64_t date_and_time;
 	u32 packet;
+	static const char * const variant_strs[] = {
+		"unknown",
+		"start",
+		"start_x",
+		"start_db",
+		"start_cd",
+	};
+	const char *variant_str = "cmd unsupported";
+	u32 variant;
 	int ret = rpi_firmware_property(fw,
 					RPI_FIRMWARE_GET_FIRMWARE_REVISION,
 					&packet, sizeof(packet));
 
-	if (ret == 0) {
-		struct tm tm;
+	if (ret)
+		return;
 
-		time_to_tm(packet, 0, &tm);
+	/* This is not compatible with y2038 */
+	date_and_time = packet;
 
-		dev_info(fw->cl.dev,
-			 "Attached to firmware from %04ld-%02d-%02d %02d:%02d\n",
-			 tm.tm_year + 1900, tm.tm_mon + 1, tm.tm_mday,
-			 tm.tm_hour, tm.tm_min);
+	ret = rpi_firmware_property(fw, RPI_FIRMWARE_GET_FIRMWARE_VARIANT,
+				    &variant, sizeof(variant));
+
+	if (!ret) {
+		if (variant >= ARRAY_SIZE(variant_strs))
+			variant = 0;
+		variant_str = variant_strs[variant];
+	}
+
+	dev_info(fw->cl.dev,
+		 "Attached to firmware from %ptT, variant %s\n",
+		 &date_and_time, variant_str);
+}
+
+static void
+rpi_firmware_print_firmware_hash(struct rpi_firmware *fw)
+{
+	u32 hash[5];
+	int ret = rpi_firmware_property(fw,
+					RPI_FIRMWARE_GET_FIRMWARE_HASH,
+					hash, sizeof(hash));
+
+	if (ret)
+		return;
+
+	dev_info(fw->cl.dev,
+		 "Firmware hash is %08x%08x%08x%08x%08x\n",
+		 hash[0], hash[1], hash[2], hash[3], hash[4]);
+}
+
+static void
+rpi_register_hwmon_driver(struct device *dev, struct rpi_firmware *fw)
+{
+	u32 packet;
+	int ret = rpi_firmware_property(fw, RPI_FIRMWARE_GET_THROTTLED,
+					&packet, sizeof(packet));
+
+	if (ret)
+		return;
+
+	rpi_hwmon = platform_device_register_data(dev, "raspberrypi-hwmon",
+						  -1, NULL, 0);
+
+	if (!IS_ERR_OR_NULL(rpi_hwmon)) {
+		if (devm_device_add_group(dev, &rpi_firmware_dev_group))
+			dev_err(dev, "Failed to create get_trottled attr\n");
 	}
 }
 
+static void rpi_register_clk_driver(struct device *dev)
+{
+	struct device_node *firmware;
+
+	/*
+	 * Earlier DTs don't have a node for the firmware clocks but
+	 * rely on us creating a platform device by hand. If we do
+	 * have a node for the firmware clocks, just bail out here.
+	 */
+	firmware = of_get_compatible_child(dev->of_node,
+					   "raspberrypi,firmware-clocks");
+	if (firmware) {
+		of_node_put(firmware);
+		return;
+	}
+
+	rpi_clk = platform_device_register_data(dev, "raspberrypi-clk",
+						-1, NULL, 0);
+}
+
+static void rpi_firmware_delete(struct kref *kref)
+{
+	struct rpi_firmware *fw = container_of(kref, struct rpi_firmware,
+					       consumers);
+
+	mbox_free_channel(fw->chan);
+	kfree(fw);
+}
+
+void rpi_firmware_put(struct rpi_firmware *fw)
+{
+	kref_put(&fw->consumers, rpi_firmware_delete);
+}
+EXPORT_SYMBOL_GPL(rpi_firmware_put);
+
 static int rpi_firmware_probe(struct platform_device *pdev)
 {
 	struct device *dev = &pdev->dev;
 	struct rpi_firmware *fw;
-	int ret;
 
-	ret = devm_device_add_group(dev, &rpi_firmware_dev_group);
-	if (ret)
-		return ret;
-
-	fw = devm_kzalloc(dev, sizeof(*fw), GFP_KERNEL);
+	/*
+	 * Memory will be freed by rpi_firmware_delete() once all users have
+	 * released their firmware handles. Don't use devm_kzalloc() here.
+	 */
+	fw = kzalloc(sizeof(*fw), GFP_KERNEL);
 	if (!fw)
 		return -ENOMEM;
 
@@ -382,26 +383,41 @@ static int rpi_firmware_probe(struct platform_device *pdev)
 	}
 
 	init_completion(&fw->c);
-	INIT_DELAYED_WORK(&fw->get_throttled_poll_work, get_throttled_poll);
+	kref_init(&fw->consumers);
 
 	platform_set_drvdata(pdev, fw);
 	g_pdev = pdev;
 
 	rpi_firmware_print_firmware_revision(fw);
-
-	schedule_delayed_work(&fw->get_throttled_poll_work, 0);
+	rpi_firmware_print_firmware_hash(fw);
+	rpi_register_hwmon_driver(dev, fw);
+	rpi_register_clk_driver(dev);
 
 	return 0;
 }
 
+static void rpi_firmware_shutdown(struct platform_device *pdev)
+{
+	struct rpi_firmware *fw = platform_get_drvdata(pdev);
+
+	if (!fw)
+		return;
+
+	rpi_firmware_property(fw, RPI_FIRMWARE_NOTIFY_REBOOT, NULL, 0);
+}
+
 static int rpi_firmware_remove(struct platform_device *pdev)
 {
 	struct rpi_firmware *fw = platform_get_drvdata(pdev);
 
-	cancel_delayed_work_sync(&fw->get_throttled_poll_work);
-	mbox_free_channel(fw->chan);
+	platform_device_unregister(rpi_hwmon);
+	rpi_hwmon = NULL;
+	platform_device_unregister(rpi_clk);
+	rpi_clk = NULL;
 	g_pdev = NULL;
 
+	rpi_firmware_put(fw);
+
 	return 0;
 }
 
@@ -409,16 +425,32 @@ static int rpi_firmware_remove(struct platform_device *pdev)
  * rpi_firmware_get - Get pointer to rpi_firmware structure.
  * @firmware_node:    Pointer to the firmware Device Tree node.
  *
+ * The reference to rpi_firmware has to be released with rpi_firmware_put().
+ *
  * Returns NULL is the firmware device is not ready.
  */
 struct rpi_firmware *rpi_firmware_get(struct device_node *firmware_node)
 {
-	struct platform_device *pdev = g_pdev;
+	struct platform_device *pdev = of_find_device_by_node(firmware_node);
+	struct rpi_firmware *fw;
 
 	if (!pdev)
 		return NULL;
 
-	return platform_get_drvdata(pdev);
+	fw = platform_get_drvdata(pdev);
+	if (!fw)
+		goto err_put_device;
+
+	if (!kref_get_unless_zero(&fw->consumers))
+		goto err_put_device;
+
+	put_device(&pdev->dev);
+
+	return fw;
+
+err_put_device:
+	put_device(&pdev->dev);
+	return NULL;
 }
 EXPORT_SYMBOL_GPL(rpi_firmware_get);
 
@@ -434,6 +466,7 @@ static struct platform_driver rpi_firmware_driver = {
 		.of_match_table = rpi_firmware_of_match,
 	},
 	.probe		= rpi_firmware_probe,
+	.shutdown	= rpi_firmware_shutdown,
 	.remove		= rpi_firmware_remove,
 };
 
@@ -457,7 +490,7 @@ out2:
 out1:
 	return ret;
 }
-subsys_initcall(rpi_firmware_init);
+core_initcall(rpi_firmware_init);
 
 static void __init rpi_firmware_exit(void)
 {
